package client;

import net.jini.discovery.*;
import net.jini.core.lookup.*;
import net.jini.core.entry.*;
import net.jini.lookup.entry.*;
import net.jini.entry.*;
//import com.sun.jini.lookup.entry.*;
import java.rmi.*;

import service.TimeServer;

public class TimeClient {
        public static void main(String args[]) throws Exception {
                System.out.println("Time client");

                // install sec mgr so that we'd be able to load remote classes
                System.setSecurityManager(new RMISecurityManager());

                // Create the lookup discoverer
                // Since we specified no String[0], no discovery is done yet
                LookupDiscovery ld = new LookupDiscovery(new String[0]);

                // Register a listener
                ld.addDiscoveryListener(new MyDiscoveryListener());

                // now actually start the discovery process.
                // This wasn't really necessary, since LookupDiscovery would
                // have told us about any previously discovered lookup services
                // anyway.
                ld.setGroups(LookupDiscovery.ALL_GROUPS);

                // and make sure we don't die
                Thread.currentThread().sleep(1000000000L);
        }

        public static void dumpRegistrar(ServiceRegistrar reg)
            throws RemoteException, NoSuchFieldException {
                System.out.println("Registrar " + reg.getLocator().toString());

                System.out.println();
                System.out.println("Getting entry classes:");

                // make a ServiceTemplate that will much all
                ServiceTemplate st = new ServiceTemplate(null, null, null);
                Class services[] = reg.getEntryClasses(st);

                System.out.println("Got " + services.length + " attribute classes");
                for (int i = 0; i < services.length; i++) {
                        if (services[i] != null)
                            System.out.println("Service " + i + " class is " + services[i].getName());
                }

                System.out.println("Getting service class types:");
                services = reg.getServiceTypes(st, "");
                System.out.println("Got " + services.length + " classes");
                for (int i = 0; i < services.length; i++) {
                        if (services[i] != null) System.out.println("Service " + i + " class is " + services[i].getName());
                }

                // Create an empty Name template
                Name nameEntry = new Name(null);
                Entry entries[] = new Entry[] {nameEntry};

                // make sure we know how to deal with non-existent fields
//                dumpEntryField(reg, entries, "fake");

                dumpEntryField(reg, entries, "name");

                // Create an empty ServiceInfo template
                entries = new Entry[] {new ServiceInfo(null, null, null, null, null, null)};
               
                dumpEntryField(reg, entries, "manufacturer");
                dumpEntryField(reg, entries, "model");
                dumpEntryField(reg, entries, "name");
                dumpEntryField(reg, entries, "serialNumber");
                dumpEntryField(reg, entries, "vendor");
                dumpEntryField(reg, entries, "version");


                // now get specified services:
                System.out.println();
                System.out.println("Looking for up to 10 services that implement service.TimeServer: ");
                entries = new Entry[] {nameEntry};
                ServiceMatches sm = reg.lookup(new ServiceTemplate(null, new Class[] {service.TimeServer.class}, null), 10);
                System.out.println("Got back " + sm.totalMatches + " services");

                for (int i = 0; i < sm.totalMatches; i++) {
                    try {
                        TimeServer times = (TimeServer) sm.items[i].service;
                        if (times == null) {
                            System.out.println("Service " + i + " could not be deserialized");
                        } else {
                            System.out.println("Time Server " + i);
                            System.out.println("Got time " + times.getTime() + " from host " + times.getHost());
                            System.out.println("Text message is: " + times.getText());
                        }
                    } catch (Exception e) {
                        System.out.println("Excepion while getting time service: " + e.getMessage());
                        e.printStackTrace();
                    }
                }

        }
        
        static void dumpEntryField(ServiceRegistrar reg, Entry entries[], String fieldName)
            throws RemoteException {
                try {
                        // template
                        System.out.println();
                        System.out.println("Getting " + fieldName + " field from " + entries[0].getClass().getName() + " entry:");
                        ServiceTemplate st = new ServiceTemplate(null, null, entries);
                        Object values[] = reg.getFieldValues(st, 0, fieldName);
                        if (values == null) {
                                System.out.println("No values present");
                        } else for (int i = 0; i < values.length; i++) {
                                System.out.println("Value " + i + " is " + values[i]);
                        }
                } catch (NoSuchFieldException e) {
                        System.out.println("No such field: " + fieldName);
                }
        }

}

class MyDiscoveryListener implements DiscoveryListener {
                        public void discovered(DiscoveryEvent e) {
                                dumpEvent("discovered: ", e);
                        }

                        public void discarded(DiscoveryEvent e) {
                                dumpEvent("discarded: ", e);
                        }

                        // Let's print info on the registrars
                        public void dumpEvent(String s, DiscoveryEvent e) {
                                try {
                                        System.out.println(s);
                                        ServiceRegistrar regs[] = e.getRegistrars();
                                        for (int i = 0; i < regs.length; i++) {
                                                TimeClient.dumpRegistrar(regs[i]);
                                        }
                                } catch (Exception z) {
                                        System.out.println("Caught ex while printing registrar: " + z.getMessage());
                                        z.printStackTrace();
                                }
                        }
}
